﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;

namespace RH.EmptyAdminRecycleBin.Layouts.RH.EmptyAdminRecycleBin
{
	public partial class EmptyAdminRecycleBin : LayoutsPageBase
	{
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			try
			{
				if (!Site.RootWeb.DoesUserHavePermissions(SPBasePermissions.ManageWeb))
					throw new UnauthorizedAccessException("You do not have the 'MangeWeb' permissions on this sitecollection.");

				// get items from the recyclebin and show statistics
				int itemCount;
				long overalSize;
				GetRecycleBinStorageInfo(out itemCount, out overalSize);
				ItemCount.Text = string.Format("{0} items ({1})", itemCount, SPUtility.FormatSize(overalSize));
			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
		}

		private void GetRecycleBinStorageInfo(out int itemCount, out long overalSize)
		{
			itemCount = 0;
			overalSize = 0;
			SPRecycleBinItemCollectionPosition itemcollectionPosition = null;
			do
			{
				SPRecycleBinQuery query = CreateQuery(itemcollectionPosition);
				SPRecycleBinItemCollection recycleBinItems = Site.GetRecycleBinItems(query);
				itemcollectionPosition = recycleBinItems.ItemCollectionPosition;
				// get itemCount
				itemCount += recycleBinItems.Count;
				// get overalSize
				overalSize += recycleBinItems.Cast<SPRecycleBinItem>().Sum(item2 => item2.Size);
			} while (itemcollectionPosition != null);
		}

		protected void EmptyButtonClick(object sender, EventArgs e)
		{
			try
			{
				if (!Site.RootWeb.DoesUserHavePermissions(SPBasePermissions.ManageWeb))
					throw new UnauthorizedAccessException("You do not have the 'MangeWeb' permissions on this sitecollection.");

				var successfull = EmptyRecycleBin();
				if (successfull)
				{
					var message = SPContext.Current.Web.CurrentUser.LoginName + " deleted all items from the Admin RecycleBin in " + Site.Url;
					UlsLogging.Write(TraceSeverity.Monitorable, message);
				}
			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
		}

		private bool EmptyRecycleBin()
		{
			using (var operation = new SPLongOperation(this))
			{
				operation.Begin();

				try
				{
					// delete all items in pages of 200 items
					SPRecycleBinItemCollectionPosition itemcollectionPosition = null;
					do
					{
						SPRecycleBinQuery query = CreateQuery(itemcollectionPosition);
						SPRecycleBinItemCollection recycleBinItems = Site.GetRecycleBinItems(query);
						itemcollectionPosition = recycleBinItems.ItemCollectionPosition;
						for (int i = 0; i < recycleBinItems.Count; i++)
						{
							recycleBinItems[i].Delete();
						}
					} while (itemcollectionPosition != null);

					operation.End("/_layouts/RH.EmptyAdminRecycleBin/CloseModalDialog.html", SPRedirectFlags.Default, Context, null);
				}
				catch (Exception ex)
				{
					UlsLogging.Write(TraceSeverity.Unexpected, ex.ToString());
					SPUtility.TransferToErrorPage(ex.Message);
					return false;
				}
				return true;
			}
		}

		private static SPRecycleBinQuery CreateQuery(SPRecycleBinItemCollectionPosition page)
		{
			var query = new SPRecycleBinQuery
			            	{
			            		RowLimit = 200,
									ItemState = SPRecycleBinItemState.SecondStageRecycleBin,
			            		OrderBy = SPRecycleBinOrderBy.Default,
			            		ItemCollectionPosition = page ?? SPRecycleBinItemCollectionPosition.FirstPage
			            	};
			return query;
		}

		private void HandleException(Exception ex)
		{
			try
			{
				UlsLogging.Write(TraceSeverity.Unexpected, ex.ToString());
				Controls.AddAt(Controls.Count, new Label {CssClass = "ms-error", Text = ex.Message});
			}
			catch (Exception e)
			{
				Trace.Write(e.ToString());
				Page.Response.Write(ex.Message);
			}
		}
	}
}
